@@ -29,15 +29,21 @@
 #endif
 
 #include <grilo.h>
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 
+#ifdef GOA_ENABLED
+#define GOA_API_IS_SUBJECT_TO_CHANGE
+#include <goa/goa.h>
+#endif
+
 #include "grl-flickr.h"
 #include "gflickr.h"
 
-#define GRL_FLICKR_SOURCE_GET_PRIVATE(object)                           \
-  (G_TYPE_INSTANCE_GET_PRIVATE((object),                                \
-                               GRL_FLICKR_SOURCE_TYPE,                  \
+#define GRL_FLICKR_SOURCE_GET_PRIVATE(object)                          \
+  (G_TYPE_INSTANCE_GET_PRIVATE((object),                               \
+                               GRL_FLICKR_SOURCE_TYPE,                 \
                                GrlFlickrSourcePrivate))
 
 /* --------- Logging  -------- */
@@ -54,11 +60,13 @@ GRL_LOG_DOMAIN(flickr_log_domain);
 
 #define PUBLIC_SOURCE_ID   "grl-flickr"
 #define PUBLIC_SOURCE_NAME "Flickr"
-#define PUBLIC_SOURCE_DESC "A source for browsing and searching Flickr photos"
+#define PUBLIC_SOURCE_DESC _("A source for browsing and searching Flickr photos")
 
 #define PERSONAL_SOURCE_ID "grl-flickr-%s"
-#define PERSONAL_SOURCE_NAME "%s's Flickr"
-#define PERSONAL_SOURCE_DESC "A source for browsing and searching %s' flickr photos"
+/* "%s" is a full user name, like "John Doe" */
+#define PERSONAL_SOURCE_NAME _("%s's Flickr")
+/* "%s" is a full user name, like "John Doe" */
+#define PERSONAL_SOURCE_DESC _("A source for browsing and searching %s' flickr photos")
 
 typedef struct {
   GrlSource *source;
@@ -88,7 +96,8 @@ static GrlFlickrSource *grl_flickr_source_public_new (const gchar *flickr_api_ke
 static void grl_flickr_source_personal_new (GrlPlugin *plugin,
                                             const gchar *flickr_api_key,
                                             const gchar *flickr_secret,
-                                            const gchar *flickr_token);
+                                            const gchar *flickr_token,
+                                            const gchar *token_secret);
 
 static void grl_flickr_source_finalize (GObject *object);
 
@@ -108,6 +117,10 @@ static void grl_flickr_source_resolve (GrlSource *source,
 static void grl_flickr_source_search (GrlSource *source,
                                       GrlSourceSearchSpec *ss);
 
+#ifdef GOA_ENABLED
+static GList *grl_flickr_get_goa_multiple_config (GrlPlugin *plugin, gboolean public);
+#endif
+
 /* =================== Flickr Plugin  =============== */
 
 gboolean
@@ -115,40 +128,74 @@ grl_flickr_plugin_init (GrlRegistry *registry,
                         GrlPlugin *plugin,
                         GList *configs)
 {
-  gchar *flickr_key;
-  gchar *flickr_secret;
-  gchar *flickr_token;
+  gchar *flickr_key           = NULL;
+  gchar *flickr_secret        = NULL;
+  gchar *flickr_token         = NULL;
+  gchar *flickr_token_secret  = NULL;
+
   GrlConfig *config;
   gboolean public_source_created = FALSE;
 
   GRL_LOG_DOMAIN_INIT (flickr_log_domain, "flickr");
 
+#ifdef GOA_ENABLED
+  GRL_DEBUG ("GOA enabled");
+  gboolean create_public_from_goa = FALSE;
+#endif
+
   GRL_DEBUG ("flickr_plugin_init");
 
-  if (!configs) {
+  /* Initialize i18n */
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+
+ if (configs == NULL) {
+#ifdef GOA_ENABLED
+    GRL_DEBUG ("No user config passed.");
+    create_public_from_goa = TRUE;
+#else
     GRL_INFO ("Configuration not provided! Plugin not loaded");
     return FALSE;
+#endif /* GOA_ENABLED */
   }
 
+#ifdef GOA_ENABLED
+  /* When is GOA enabled, add all configs from GOA too */
+  GList *goa_config = grl_flickr_get_goa_multiple_config (plugin,
+                                                          create_public_from_goa);
+
+  if (goa_config == NULL)
+  {
+    GRL_INFO ("Cannot get flickr sources from GOA.");
+  }
+  else
+  {
+    configs = g_list_concat (configs, goa_config);
+  }
+#endif /* GOA_ENABLED */
+
   while (configs) {
     config = GRL_CONFIG (configs->data);
 
     flickr_key = grl_config_get_api_key (config);
     flickr_token = grl_config_get_api_token (config);
+    flickr_token_secret = grl_config_get_api_token_secret (config);
     flickr_secret = grl_config_get_api_secret (config);
 
     if (!flickr_key || !flickr_secret) {
       GRL_INFO ("Required API key or secret configuration not provdied. "
                 " Plugin not loaded");
-    } else if (flickr_token) {
+    } else if (flickr_token && flickr_token_secret) {
       grl_flickr_source_personal_new (plugin,
                                       flickr_key,
                                       flickr_secret,
-                                      flickr_token);
+                                      flickr_token,
+                                      flickr_token_secret);
     } else if (public_source_created) {
       GRL_WARNING ("Only one public source can be created");
     } else {
-      GrlFlickrSource *source = grl_flickr_source_public_new (flickr_key, flickr_secret);
+      GrlFlickrSource *source = grl_flickr_source_public_new (flickr_key,
+                                                              flickr_secret);
       public_source_created = TRUE;
       grl_registry_register_source (registry,
                                     plugin,
@@ -162,6 +209,8 @@ grl_flickr_plugin_init (GrlRegistry *registry,
       g_free (flickr_token);
     if (flickr_secret != NULL)
       g_free (flickr_secret);
+    if (flickr_token_secret != NULL)
+      g_free (flickr_token_secret);
 
     configs = g_list_next (configs);
   }
@@ -183,7 +232,7 @@ grl_flickr_source_public_new (const gchar *flickr_api_key,
 {
   GrlFlickrSource *source;
 
-  GRL_DEBUG ("grl_flickr_source_new");
+  GRL_DEBUG ("grl_flickr_public_source_new");
 
   source = g_object_new (GRL_FLICKR_SOURCE_TYPE,
                          "source-id", PUBLIC_SOURCE_ID,
@@ -191,7 +240,8 @@ grl_flickr_source_public_new (const gchar *flickr_api_key,
                          "source-desc", PUBLIC_SOURCE_DESC,
                          "supported-media", GRL_MEDIA_TYPE_IMAGE,
                          NULL);
-  source->priv->flickr = g_flickr_new (flickr_api_key, flickr_secret, NULL);
+  source->priv->flickr = g_flickr_new (flickr_api_key, flickr_secret,
+                                       NULL, NULL);
 
   return source;
 }
@@ -200,12 +250,18 @@ static void
 grl_flickr_source_personal_new (GrlPlugin *plugin,
                                 const gchar *flickr_api_key,
                                 const gchar *flickr_secret,
-                                const gchar *flickr_token)
+                                const gchar *flickr_token,
+                                const gchar *flickr_token_secret)
 {
   GFlickr *f;
 
-  f = g_flickr_new (flickr_api_key, flickr_secret, flickr_token);
-  g_flickr_auth_checkToken (f, flickr_token, token_info_cb, (gpointer) plugin);
+  GRL_DEBUG ("grl_flickr_personal_source_new");
+
+  f = g_flickr_new (flickr_api_key, flickr_secret,
+                    flickr_token, flickr_token_secret);
+
+  g_flickr_auth_checkToken (f, flickr_token, token_info_cb,
+                            (gpointer) plugin);
 }
 
 static void
@@ -514,7 +570,9 @@ photosetsphotos_cb (GFlickr *f, GList *photolist, gpointer user_data)
 
   while (photolist && od->count) {
     media_type = g_hash_table_lookup (photolist->data, "photo_media");
-    if (strcmp (media_type, "photo") == 0) {
+    if (media_type == NULL) {
+      media = grl_media_new ();
+    } else if (strcmp (media_type, "photo") == 0) {
       media = grl_media_image_new ();
     } else {
       media = grl_media_video_new ();
@@ -535,7 +593,8 @@ photosetsphotos_cb (GFlickr *f, GList *photolist, gpointer user_data)
   if (od->count) {
     od->offset = 0;
     od->page++;
-    g_flickr_photosets_getPhotos (f, od->text, od->page, photosetsphotos_cb, od);
+    g_flickr_photosets_getPhotos (f, od->text, od->page,
+                                  photosetsphotos_cb, od);
   } else {
     g_slice_free (OperationData, od);
   }
@@ -549,7 +608,8 @@ gettags_cb (GFlickr *f, GList *taglist, gpointer user_data)
   gint count;
 
   /* Go to offset element */
-  taglist = g_list_nth (taglist, grl_operation_options_get_skip (bs->options));
+  taglist = g_list_nth (taglist,
+                        grl_operation_options_get_skip (bs->options));
 
   /* No more elements can be sent */
   if (!taglist) {
@@ -579,6 +639,106 @@ gettags_cb (GFlickr *f, GList *taglist, gpointer user_data)
   }
 }
 
+#ifdef GOA_ENABLED
+static GList *
+grl_flickr_get_goa_multiple_config (GrlPlugin *plugin, gboolean public)
+{
+  GList *tmp;
+  GList *list = NULL;
+  GError *error = NULL;
+  GrlConfig *conf = NULL;
+  GList *configs = NULL;
+
+  gchar *access_token;
+  gchar *token_secret;
+
+  gboolean public_created = FALSE;
+
+  GoaAccount *acc = NULL;
+  GoaOAuthBased *oauth = NULL;
+  GoaClient *cl = goa_client_new_sync (NULL, &error);
+
+  if (error != NULL)
+  {
+    GRL_ERROR ("%s\n", error->message);
+    return NULL;
+  }
+
+  list = goa_client_get_accounts (cl);
+  tmp = g_list_first (list);
+
+  /* find flickr one's and get tokens */
+  while (tmp != NULL)
+  {
+    acc = goa_object_peek_account (tmp->data);
+
+    if (strcmp (goa_account_get_provider_type (acc), "flickr") == 0)
+    {
+      oauth = goa_object_peek_oauth_based (tmp->data);
+
+      if (oauth != NULL)
+      {
+        conf = grl_config_new (grl_plugin_get_id (plugin),
+                               NULL);
+
+        /* Consumer data */
+        grl_config_set_api_key (conf,
+                                goa_oauth_based_get_consumer_key (oauth));
+        grl_config_set_api_secret (conf,
+                                   goa_oauth_based_get_consumer_secret (oauth));
+
+        /* if public == TRUE, create one public source */
+        if (public == TRUE && public_created == FALSE)
+        {
+          configs = g_list_append (configs, conf);
+          public_created = TRUE;
+
+          continue; /* Use this personal source again, but this time with tokens */
+        }
+
+        /* Get Access Token */
+        if (! goa_oauth_based_call_get_access_token_sync (oauth,
+                                                          &access_token,
+                                                          &token_secret,
+                                                          NULL, NULL,
+                                                          &error))
+        {
+
+          /* No access token doesn't mean error */
+          GRL_INFO ("Access token: %s\n", error->message);
+          g_error_free (error);
+        }
+        else
+        {
+          grl_config_set_api_token (conf, access_token);
+          grl_config_set_api_token_secret (conf, token_secret);
+
+          if (access_token != NULL)
+          {
+            g_free(access_token);
+            access_token = NULL;
+          }
+
+          if (token_secret != NULL)
+          {
+            g_free(token_secret);
+            token_secret = NULL;
+          }
+        }
+
+        configs = g_list_append (configs, conf);
+      }
+    }
+    tmp = g_list_next (tmp);
+  }
+
+  g_object_unref (cl);
+  g_list_free_full (list, g_object_unref);
+
+  return configs;
+}
+#endif /* FLICKR_GOA_ENABLED */
+
 /* ================== API Implementation ================ */
 
 static const GList *
@@ -614,7 +774,9 @@ grl_flickr_source_public_browse (GrlSource *source,
   if (!container_id) {
     /* Get hot tags list. List is limited up to HOTLIST_MAX tags */
     if (skip >= HOTLIST_MAX) {
-      bs->callback (bs->source, bs->operation_id, NULL, 0, bs->user_data, NULL);
+      bs->callback (bs->source, bs->operation_id,
+                    NULL, 0,
+                    bs->user_data, NULL);
     } else {
       request_size = CLAMP (skip + count, 1, HOTLIST_MAX);
       g_flickr_tags_getHotList (f, request_size, gettags_cb, bs);
@@ -683,7 +845,8 @@ grl_flickr_source_personal_browse (GrlSource *source,
     od->count = count;
     od->operation_id = bs->operation_id;
 
-    g_flickr_photosets_getPhotos (f, container_id, od->page, photosetsphotos_cb, od);
+    g_flickr_photosets_getPhotos (f, container_id, od->page,
+                                  photosetsphotos_cb, od);
   }
 }
 
@@ -705,7 +868,8 @@ grl_flickr_source_resolve (GrlSource *source,
   const gchar *id;
 
   if (!rs->media || (id = grl_media_get_id (rs->media)) == NULL) {
-    rs->callback (rs->source, rs->operation_id, rs->media, rs->user_data, NULL);
+    rs->callback (rs->source, rs->operation_id,
+                  rs->media, rs->user_data, NULL);
     return;
   }
 
