@@ -228,7 +228,7 @@ fill_list_filtered (JsonArray *array,
   FilterClosure *closure = (FilterClosure *) user_data;
 
   if (closure->filter == NULL || closure->filter (element)) {
-    closure->list = g_list_prepend (closure->list, element);
+    closure->list = g_list_prepend (closure->list, json_node_copy (element));
   }
 }
 
@@ -529,12 +529,11 @@ grl_tmdb_request_get (GrlTmdbRequest *self,
                       const char *path)
 {
   JsonNode *node;
+  JsonNode *element;
   GError *error = NULL;
   GValue *value = NULL;
   JsonArray *values;
 
-  value = g_new0 (GValue, 1);
-
   node = json_path_query (path,
                           json_parser_get_root (self->priv->parser),
                           &error);
@@ -546,7 +545,12 @@ grl_tmdb_request_get (GrlTmdbRequest *self,
   }
 
   values = json_node_get_array (node);
-  json_node_get_value (json_array_get_element (values, 0), value);
+  element = json_array_get_element (values, 0);
+
+  if (JSON_NODE_HOLDS_VALUE (element)) {
+    value = g_new0 (GValue, 1);
+    json_node_get_value (element, value);
+  }
 
   json_node_free (node);
 
@@ -614,7 +618,7 @@ grl_tmdb_request_get_string_list_with_filter (GrlTmdbRequest *self,
 
   get_list_with_filter (self, path, &closure);
 
-  return closure.list;
+  return g_list_reverse (closure.list);
 }
 
 /**
@@ -640,3 +644,30 @@ grl_tmdb_request_get_uri (GrlTmdbRequest *self)
 {
   return self->priv->uri;
 }
+
+/**
+ * grl_tmdb_request_detail_to_string:
+ * @detail: A #GrlTmdbRequestDetail
+ * Returns: A description of the detail or %NULL for invalid details.
+ */
+const char *
+grl_tmdb_request_detail_to_string (GrlTmdbRequestDetail detail)
+{
+  switch (detail) {
+    case GRL_TMDB_REQUEST_DETAIL_MOVIE:
+      return "generic details";
+    case GRL_TMDB_REQUEST_DETAIL_MOVIE_CAST:
+      return "casts";
+    case GRL_TMDB_REQUEST_DETAIL_MOVIE_IMAGES:
+      return "images";
+    case GRL_TMDB_REQUEST_DETAIL_MOVIE_KEYWORDS:
+      return "keywords";
+    case GRL_TMDB_REQUEST_DETAIL_MOVIE_RELEASE_INFO:
+      return "release information";
+    case GRL_TMDB_REQUEST_DETAIL_COUNT:
+      break;
+  }
+
+  g_warn_if_reached ();
+  return NULL;
+}
